/**
 * Class patternMatch - Small program for pattern matching 
 * 
 * @author: Bernd J. Krmer, Hans-Friedrich Ktter, Thomas H. A. Dammann
 * @version $Revision: 1.1 $ --- $Date: 2001/08/23 14:07:25 $
 */


public class PatternMatch 
{ 

    public final int        MAX_LENGTH = 20;        // Maximal length of strings (text and pattern)


                                                    // Textstring, in which the pattern should be searched
    public char             t[] = new char[MAX_LENGTH];
                                                    // Pattern
    public char             p[] = new char[MAX_LENGTH]; 
    public int              t_len,                  // Length of the textstring
                            p_len;                  // Length of the pattern
    public boolean          patternFound = false;   // Stores whether the pattern was found

   /**
    * Realizes the algorithm shown at the course page "Searching in Arrays".
    * The difference to the course is that the user can input freely text
    * string and pattern. The number of comparisons is computed and outputed.
    */

    public void patternMatch() 
    {
        int     i, j;                   // Loop variables
        boolean patternFound = false;   // Stores whether the pattern was found
        int     countCom = 0;           // Counts the comparisons
        int     count;                  // Counts the number of characters matches the pattern
     
        getStrings();                   // Get t and p
        for (i = 0; i <= (t_len - p_len); i++) 
        {
            count = 0;                  // Reset the counter for the pattern if a new position is proved
            for (j = 0; j < p_len; j++) 
            { 
                countCom++;             // Count this comparison
                if (t[i + j] == p[j]) 
                { 
                    count++;            // This character matches the pattern
                    if (count == p_len) // Have all characters matched the pattern? 
                    { 
                        // Yes. Then a pattern match occurs at text string position i + 1
                        // Show the user the position of the pattern
                        System.out.println("Pattern found at: " + (i + 1) ); 
                        patternFound = true;    // Remember that the pattern was found at least once
                    }
                }
            }
        }
        if (!patternFound)
        {
            // Inform the user that the pattern wasn't found
            System.out.println("Text string doesn't contain pattern");
        }
        System.out.println("Number of comparisons: " + countCom);
    }

   /**
    * Same as "patternMatch", but optimized as shown in the animation 2 at the
    * course page "Searching in Arrays".
    * You should see that you need a lesser number of comparisons than in
    * "patternMatch". It isn't longer nessessary to count the number of
    * character match at each position of the text string (variable count).
    */
    public void patternMatchModified() 
    {
        int     i, j;                   // Loop variables
        boolean patternFound = false;   // Stores whether the pattern was found
        int     countCom = 0;           // Counts the comparisons

     
        getStrings();                   // Get t and p
        for (i = 0; i <= (t_len - p_len); i++) 
        {
            for (j = 0; j < p_len; j++) 
            { 
                countCom++;             // Count this comparison
                if (t[i + j] == p[j]) 
                { 
                    if (j == p_len - 1) 
                    { 
                        // Show the user the position of the pattern
                        System.out.println("Pattern found at: " + (i + 1) ); 
                        patternFound = true;    // Remember that the pattern was found at least once
                    }
                }
                else                    // This is the modification
                {
                    // If the pattern doesn't match the text at one position,
                    //  shift the pattern right by one position, i. e. 
                    //  leave the inner loop
                    break;
                }
            }
        }
        if (!patternFound)
        {
            // Inform the user that the pattern wasn't found
            System.out.println("Text string doesn't contain pattern");
        }
        System.out.println("Number of comparisons: " + countCom);
    }
    
   /**
    * Prompt the user to input the text string an the pattern. Puts this
    * strings to the character arrays t resp. p
    */
    private void getStrings()
    {
        StringBuffer    strBuf;                 // String used to input the arrays

        try
        {
            // Enter text string and assign it to character array t
            strBuf = new StringBuffer(Console.readString("Enter text string (max. "
                                        + MAX_LENGTH + " are evaluated): "));
            t_len = strBuf.length();
            if (t_len > MAX_LENGTH)
            {
                t_len = MAX_LENGTH;
            }
            strBuf.getChars(0,t_len,t,0);

            // Enter pattern string and assign it to character array p
            strBuf = new StringBuffer(Console.readString("Enter pattern string (max. "
                                        + MAX_LENGTH + " are evaluated): "));
            p_len = strBuf.length();
            if (p_len > MAX_LENGTH)
            {
                p_len = MAX_LENGTH;
            }
            strBuf.getChars(0,p_len,p,0);
        }
        catch (StringIndexOutOfBoundsException e)
        {
            // Do nothing
        }

    }
}
